@@ -30,6 +30,7 @@
 #include <kconfiggroup.h>
 #include <ksharedconfig.h>
 
+#include "rsiglobals.h"
 #include "rsistats.h"
 
 RSITimer::RSITimer( QObject *parent ) : QThread( parent )
@@ -40,9 +41,9 @@ RSITimer::RSITimer( QObject *parent ) : QThread( parent )
     updateConfig( true );
 }
 
-RSITimer::RSITimer( RSIIdleTime* _idleTime, const QVector<int> _intervals,
-                    const bool _usePopup, const bool _useIdleTimers ) : QThread( 0 )
-    , m_idleTimeInstance( _idleTime )
+RSITimer::RSITimer( std::unique_ptr<RSIIdleTime> &&_idleTime, const QVector<int> _intervals,
+                    const bool _usePopup, const bool _useIdleTimers ) : QThread( nullptr )
+    , m_idleTimeInstance( std::move(_idleTime) )
     , m_usePopup( _usePopup )
     , m_useIdleTimers( _useIdleTimers )
     , m_intervals( _intervals )
@@ -106,7 +107,11 @@ void RSITimer::doBreakNow( const int breakTime, const bool nextBreakIsBig )
     m_state = TimerState::Resting;
     m_pauseCounter = std::unique_ptr<RSITimerCounter> { new RSITimerCounter( breakTime, breakTime, INT_MAX ) };
     m_popupCounter = nullptr;
-    RSIGlobals::instance()->NotifyBreak( true, nextBreakIsBig );
+    if ( nextBreakIsBig ) {
+        emit startLongBreak();
+    } else {
+        emit startShortBreak();
+    }
     emit updateWidget( breakTime );
     emit breakNow();
 }
@@ -120,6 +125,11 @@ void RSITimer::resetAfterBreak()
     emit updateIdleAvg( 0.0 );
     emit relax( -1, false );
     emit minimize();
+    if ( m_bigBreakCounter->isReset() ) {
+        emit endLongBreak();
+    } else {
+        emit endShortBreak();
+    }
 }
 
 // -------------------------- SLOTS ------------------------//
