@@ -23,9 +23,9 @@
 #define RSITimer_H
 
 #include <QThread>
+#include <QVector>
 #include <memory>
 
-#include "rsiglobals.h"
 #include "rsitimercounter.h"
 #include "rsiidletime.h"
 
@@ -56,6 +56,7 @@ public:
     int bigLeft() const { return m_bigBreakCounter->counterLeft(); };
 
 public slots:
+
     /**
       Reads the configuration and restarts the timer with slotRestart.
     */
@@ -162,6 +163,11 @@ signals:
      */
     void bigBreakSkipped();
 
+    void startLongBreak();
+    void endLongBreak();
+    void startShortBreak();
+    void endShortBreak();
+
 private:
     std::unique_ptr<RSIIdleTime> m_idleTimeInstance;
 
@@ -199,8 +205,8 @@ private:
     */
     void doBreakNow( const int breakTime, const bool nextBreakIsBig );
 
-    // Constructor for tests. Ownership is taken over for _idleTime.
-    RSITimer( RSIIdleTime* _idleTime, const QVector<int> _intervals, const bool _usePopup, const bool _useIdleTimers );
+    // Constructor for tests.
+    RSITimer( std::unique_ptr<RSIIdleTime> &&_idleTime, const QVector<int> _intervals, const bool _usePopup, const bool _useIdleTimers );
 };
 
 #endif
