@@ -16,6 +16,7 @@
 
 #include "rsitimer_test.h"
 
+#include "rsiglobals.h"
 #include "rsitimer.h"
 
 static constexpr int RELAX_ENDED_MAGIC_VALUE = -1;
@@ -35,14 +36,17 @@ RSITimerTest::RSITimerTest( void )
 
 void RSITimerTest::triggerSimpleTinyBreak()
 {
-    RSIIdleTimeFake* idleTime = new RSIIdleTimeFake();
-    RSITimer timer( idleTime, m_intervals, true, true );
+    std::unique_ptr<RSIIdleTimeFake> idle_time( new RSIIdleTimeFake() );
+    RSIIdleTimeFake* idle_time_ptr = idle_time.get();
+    RSITimer timer( std::move( idle_time ), m_intervals, true, true );
+
+    QSignalSpy spyEndShortBreak( &timer, SIGNAL( endShortBreak( void ) ) );
 
     // Part one, no idleness till small break.
-    QSignalSpy spy1Relax( &timer, SIGNAL( relax( int, bool ) ) );
-    QSignalSpy spy1UpdateIdleAvg( &timer, SIGNAL( updateIdleAvg( double ) ) );
+    QSignalSpy spy1Relax( &timer, SIGNAL(relax(int,bool)) );
+    QSignalSpy spy1UpdateIdleAvg( &timer, SIGNAL(updateIdleAvg(double)) );
 
-    idleTime->setIdleTime( 0 );
+    idle_time_ptr->setIdleTime( 0 );
     for ( int i = 0; i < m_intervals[TINY_BREAK_INTERVAL]; i++ ) {
         QCOMPARE( timer.m_state, RSITimer::TimerState::Monitoring );
         timer.timeout();
@@ -66,13 +70,13 @@ void RSITimerTest::triggerSimpleTinyBreak()
     }
 
     // Part two, obeying and idle as suggested.
-    QSignalSpy spy2Relax( &timer, SIGNAL( relax( int, bool ) ) );
-    QSignalSpy spy2UpdateIdleAvg( &timer, SIGNAL( updateIdleAvg( double ) ) );
-    QSignalSpy spy2Minimize( &timer, SIGNAL( minimize() ) );
+    QSignalSpy spy2Relax( &timer, SIGNAL(relax(int,bool)) );
+    QSignalSpy spy2UpdateIdleAvg( &timer, SIGNAL(updateIdleAvg(double)) );
+    QSignalSpy spy2Minimize( &timer, SIGNAL(minimize()) );
 
     for ( int i = 0; i < m_intervals[TINY_BREAK_DURATION]; i++ ) {
         QCOMPARE( timer.m_state, RSITimer::TimerState::Suggesting );
-        idleTime->setIdleTime( ( i + 1 ) * 1000 );
+        idle_time_ptr->setIdleTime( ( i + 1 ) * 1000 );
         timer.timeout();
     }
     QCOMPARE( timer.m_state, RSITimer::TimerState::Monitoring );
@@ -84,14 +88,14 @@ void RSITimerTest::triggerSimpleTinyBreak()
     }
     QList<QVariant> spy2RelaxSignals = spy2Relax.takeFirst(); // The last one is special.
     QCOMPARE( spy2RelaxSignals.at( 0 ).toInt(), RELAX_ENDED_MAGIC_VALUE );
-
-    // RSITimer owns idleTime, so not deleting it.
+    QCOMPARE( spyEndShortBreak.count(), 1 );
 }
 
 void RSITimerTest::triggerComplexTinyBreak()
 {
-    RSIIdleTimeFake* idleTime = new RSIIdleTimeFake();
-    RSITimer timer( idleTime, m_intervals, true, true );
+    std::unique_ptr<RSIIdleTimeFake> idle_time( new RSIIdleTimeFake() );
+    RSIIdleTimeFake* idle_time_ptr = idle_time.get();
+    RSITimer timer( std::move( idle_time ), m_intervals, true, true );
 
     int part1 = 10;         // Non-idle
     int part2 = 40;         // Idle
@@ -100,7 +104,7 @@ void RSITimerTest::triggerComplexTinyBreak()
     // Part 1, no idleness.
     QSignalSpy spy1Relax( &timer, SIGNAL( relax( int, bool ) ) );
     QSignalSpy spy1UpdateIdleAvg( &timer, SIGNAL( updateIdleAvg( double ) ) );
-    idleTime->setIdleTime( 0 );
+    idle_time_ptr->setIdleTime( 0 );
     for ( int i = 0; i < part1; i++ ) {
         timer.timeout();
         QCOMPARE( timer.m_state, RSITimer::TimerState::Monitoring );
@@ -109,10 +113,10 @@ void RSITimerTest::triggerComplexTinyBreak()
     QCOMPARE( spy1UpdateIdleAvg.count(), part1 );
 
     // Part 2, idle for a while.
-    QSignalSpy spy2Relax( &timer, SIGNAL( relax( int, bool ) ) );
-    QSignalSpy spy2UpdateIdleAvg( &timer, SIGNAL( updateIdleAvg( double ) ) );
+    QSignalSpy spy2Relax( &timer, SIGNAL(relax(int,bool)) );
+    QSignalSpy spy2UpdateIdleAvg( &timer, SIGNAL(updateIdleAvg(double)) );
     for ( int i = 0; i < part2; i++ ) {
-        idleTime->setIdleTime( ( i + 1 ) * 1000 );
+        idle_time_ptr->setIdleTime( ( i + 1 ) * 1000 );
         timer.timeout();
         QCOMPARE( timer.m_state, RSITimer::TimerState::Monitoring );
     }
@@ -120,32 +124,31 @@ void RSITimerTest::triggerComplexTinyBreak()
     QCOMPARE( spy2UpdateIdleAvg.count(), part2 );
 
     // Part 3, non-idle till break.
-    QSignalSpy spy3Relax( &timer, SIGNAL( relax( int, bool ) ) );
-    QSignalSpy spy3UpdateIdleAvg( &timer, SIGNAL( updateIdleAvg( double ) ) );
+    QSignalSpy spy3Relax( &timer, SIGNAL(relax(int,bool)) );
+    QSignalSpy spy3UpdateIdleAvg( &timer, SIGNAL(updateIdleAvg(double)) );
     for ( int i = 0; i < part3; i++ ) {
         QCOMPARE( timer.m_state, RSITimer::TimerState::Monitoring );
-        idleTime->setIdleTime( 0 );
+        idle_time_ptr->setIdleTime( 0 );
         timer.timeout();
     }
     QCOMPARE( timer.m_state, RSITimer::TimerState::Suggesting );
     QCOMPARE( spy3Relax.count(), 1 );
-
-    // RSITimer owns idleTime, so not deleting it.
 }
 
 void RSITimerTest::testSuspended()
 {
-    RSIIdleTimeFake* idleTime = new RSIIdleTimeFake();
-    RSITimer timer( idleTime, m_intervals, true, true );
+    std::unique_ptr<RSIIdleTimeFake> idle_time( new RSIIdleTimeFake() );
+    RSIIdleTimeFake* idle_time_ptr = idle_time.get();
+    RSITimer timer( std::move( idle_time ), m_intervals, true, true );
 
     timer.slotStop();
     QCOMPARE( timer.m_state, RSITimer::TimerState::Suspended );
 
-    QSignalSpy spy1Relax( &timer, SIGNAL( relax( int, bool ) ) );
-    QSignalSpy spy1UpdateIdleAvg( &timer, SIGNAL( updateIdleAvg( double ) ) );
+    QSignalSpy spy1Relax( &timer, SIGNAL(relax(int,bool)) );
+    QSignalSpy spy1UpdateIdleAvg( &timer, SIGNAL(updateIdleAvg(double)) );
 
     // Not idle for long enough to have a break.
-    idleTime->setIdleTime( 0 );
+    idle_time_ptr->setIdleTime( 0 );
     for ( int i = 0; i < m_intervals[TINY_BREAK_INTERVAL]; i++ ) {
         timer.timeout();
         QCOMPARE( timer.m_state, RSITimer::TimerState::Suspended );
@@ -155,24 +158,25 @@ void RSITimerTest::testSuspended()
 
     timer.slotStart();
     QCOMPARE( timer.m_state, RSITimer::TimerState::Monitoring );
-
-    // RSITimer owns idleTime, so not deleting it.
 }
 
 void RSITimerTest::triggerSimpleBigBreak()
 {
-    RSIIdleTimeFake* idleTime = new RSIIdleTimeFake();
-    RSITimer timer( idleTime, m_intervals, true, true );
+    std::unique_ptr<RSIIdleTimeFake> idle_time( new RSIIdleTimeFake() );
+    RSIIdleTimeFake* idle_time_ptr = idle_time.get();
+    RSITimer timer( std::move( idle_time ), m_intervals, true, true );
 
     int tinyBreaks = m_intervals[BIG_BREAK_INTERVAL] / ( m_intervals[TINY_BREAK_INTERVAL] + m_intervals[PATIENCE_INTERVAL] + m_intervals[TINY_BREAK_DURATION] );
     // We don't tick big pause timer during tiny breaks and patience, so it will actually happen later.
     int ticks = m_intervals[BIG_BREAK_INTERVAL] + tinyBreaks * ( m_intervals[PATIENCE_INTERVAL] + m_intervals[TINY_BREAK_DURATION] );
 
-    // Part one, no idleness till big break.
-    QSignalSpy spy1Relax( &timer, SIGNAL( relax( int, bool ) ) );
-    QSignalSpy spy1UpdateIdleAvg( &timer, SIGNAL( updateIdleAvg( double ) ) );
+    QSignalSpy spyEndLongBreak( &timer, SIGNAL( endLongBreak( void ) ) );
 
-    idleTime->setIdleTime( 0 );
+    // Part one, no idleness till big break.
+    QSignalSpy spy1Relax( &timer, SIGNAL(relax(int,bool)) );
+    QSignalSpy spy1UpdateIdleAvg( &timer, SIGNAL(updateIdleAvg(double)) );
+
+    idle_time_ptr->setIdleTime( 0 );
     for ( int i = 0; i < ticks; i++ ) {
         timer.timeout();
     }
@@ -183,32 +187,32 @@ void RSITimerTest::triggerSimpleBigBreak()
     QVERIFY2( spy1UpdateIdleAvg.count() >= m_intervals[BIG_BREAK_INTERVAL], "Failed to update the indicator regularly." );
 
     // Part two, making the big break.
-    QSignalSpy spy2Relax( &timer, SIGNAL( relax( int, bool ) ) );
+    QSignalSpy spy2Relax( &timer, SIGNAL(relax(int,bool)) );
     for ( int i = 0; i < m_intervals[BIG_BREAK_DURATION]; i++ ) {
         QCOMPARE( timer.m_state, RSITimer::TimerState::Suggesting );
-        idleTime->setIdleTime( ( i + 1 ) * 1000 );
+        idle_time_ptr->setIdleTime( ( i + 1 ) * 1000 );
         timer.timeout();
     }
     QCOMPARE( timer.m_state, RSITimer::TimerState::Monitoring );
     QCOMPARE( spy2Relax.count(), m_intervals[BIG_BREAK_DURATION] );
-
-    // RSITimer owns idleTime, so not deleting it.
+    QCOMPARE( spyEndLongBreak.count(), 1 );
 }
 
 void RSITimerTest::postponeBreak()
 {
-    RSIIdleTimeFake* idleTime = new RSIIdleTimeFake();
-    RSITimer timer( idleTime, m_intervals, true, true );
+    std::unique_ptr<RSIIdleTimeFake> idle_time( new RSIIdleTimeFake() );
+    RSIIdleTimeFake* idle_time_ptr = idle_time.get();
+    RSITimer timer( std::move( idle_time ), m_intervals, true, true );
 
     // Not idle for long enough to have a break.
-    idleTime->setIdleTime( 0 );
+    idle_time_ptr->setIdleTime( 0 );
     for ( int i = 0; i < m_intervals[TINY_BREAK_INTERVAL]; i++ ) {
         timer.timeout();
     }
     QCOMPARE( timer.m_state, RSITimer::TimerState::Suggesting );
 
-    QSignalSpy spyRelax( &timer, SIGNAL( relax( int, bool ) ) );
-    QSignalSpy spyMinimize( &timer, SIGNAL( minimize() ) );
+    QSignalSpy spyRelax( &timer, SIGNAL(relax(int,bool)) );
+    QSignalSpy spyMinimize( &timer, SIGNAL(minimize()) );
     timer.postponeBreak();
 
     QCOMPARE( timer.m_state, RSITimer::TimerState::Monitoring );
@@ -222,24 +226,23 @@ void RSITimerTest::postponeBreak()
         timer.timeout();
     }
     QCOMPARE( timer.m_state, RSITimer::TimerState::Suggesting );
-
-    // RSITimer owns idleTime, so not deleting it.
 }
 
 void RSITimerTest::screenLock()
 {
-    RSIIdleTimeFake* idleTime = new RSIIdleTimeFake();
-    RSITimer timer( idleTime, m_intervals, true, true );
+    std::unique_ptr<RSIIdleTimeFake> idle_time( new RSIIdleTimeFake() );
+    RSIIdleTimeFake* idle_time_ptr = idle_time.get();
+    RSITimer timer( std::move( idle_time ), m_intervals, true, true );
 
     // Not idle for long enough to have a break.
-    idleTime->setIdleTime( 0 );
+    idle_time_ptr->setIdleTime( 0 );
     for ( int i = 0; i < m_intervals[TINY_BREAK_INTERVAL]; i++ ) {
         timer.timeout();
     }
     QCOMPARE( timer.m_state, RSITimer::TimerState::Suggesting );
 
-    QSignalSpy spyRelax( &timer, SIGNAL( relax( int, bool ) ) );
-    QSignalSpy spyMinimize( &timer, SIGNAL( minimize() ) );
+    QSignalSpy spyRelax( &timer, SIGNAL(relax(int,bool)) );
+    QSignalSpy spyMinimize( &timer, SIGNAL(minimize()) );
     timer.slotLock();
 
     QCOMPARE( timer.m_state, RSITimer::TimerState::Monitoring );
@@ -249,24 +252,23 @@ void RSITimerTest::screenLock()
     QCOMPARE( spyMinimize.count(), 1 );
     QVERIFY2( timer.m_bigBreakCounter->counterLeft() < m_intervals[BIG_BREAK_INTERVAL],
               "Big break counter was reset on screen lock when it should have not." );
-
-    // RSITimer owns idleTime, so not deleting it.
 }
 
 void RSITimerTest::skipBreak()
 {
-    RSIIdleTimeFake* idleTime = new RSIIdleTimeFake();
-    RSITimer timer( idleTime, m_intervals, true, true );
+    std::unique_ptr<RSIIdleTimeFake> idle_time( new RSIIdleTimeFake() );
+    RSIIdleTimeFake* idle_time_ptr = idle_time.get();
+    RSITimer timer( std::move( idle_time ), m_intervals, true, true );
 
     // Not idle for long enough to have a break.
-    idleTime->setIdleTime( 0 );
+    idle_time_ptr->setIdleTime( 0 );
     for ( int i = 0; i < m_intervals[TINY_BREAK_INTERVAL]; i++ ) {
         timer.timeout();
     }
     QCOMPARE( timer.m_state, RSITimer::TimerState::Suggesting );
 
-    QSignalSpy spyRelax( &timer, SIGNAL( relax( int, bool ) ) );
-    QSignalSpy spyMinimize( &timer, SIGNAL( minimize() ) );
+    QSignalSpy spyRelax( &timer, SIGNAL(relax(int,bool)) );
+    QSignalSpy spyMinimize( &timer, SIGNAL(minimize()) );
     timer.skipBreak();
 
     QCOMPARE( timer.m_state, RSITimer::TimerState::Monitoring );
@@ -276,8 +278,111 @@ void RSITimerTest::skipBreak()
     QCOMPARE( spyMinimize.count(), 1 );
     QVERIFY2( timer.m_bigBreakCounter->counterLeft() < m_intervals[BIG_BREAK_INTERVAL],
               "Big break counter was reset on skip break when it should have not." );
+}
 
-    // RSITimer owns idleTime, so not deleting it.
+void RSITimerTest::noPopupBreak()
+{
+    std::unique_ptr<RSIIdleTimeFake> idle_time( new RSIIdleTimeFake() );
+    RSIIdleTimeFake* idle_time_ptr = idle_time.get();
+    RSITimer timer( std::move( idle_time ), m_intervals, false, true );
+
+    QSignalSpy spyStartShortBreak( &timer, SIGNAL( startShortBreak( void ) ) );
+    QSignalSpy spyEndShortBreak( &timer, SIGNAL( endShortBreak( void ) ) );
+
+    // Part one, no idleness till small break.
+    QSignalSpy spy1BreakNow( &timer, SIGNAL(breakNow()) );
+    QSignalSpy spy1UpdateWidget( &timer, SIGNAL(updateWidget(int)) );
+
+    idle_time_ptr->setIdleTime( 0 );
+    for ( int i = 0; i < m_intervals[TINY_BREAK_INTERVAL]; i++ ) {
+        QCOMPARE( timer.m_state, RSITimer::TimerState::Monitoring );
+        timer.timeout();
+    }
+
+    // Popup is disabled so straight to breaking.
+    QCOMPARE( spyStartShortBreak.count(), 1 );
+    QCOMPARE( timer.m_state, RSITimer::TimerState::Resting );
+
+    QCOMPARE( spy1BreakNow.count(), 1 );
+    QList<QVariant> spy1UpdateWidgetSignals = spy1UpdateWidget.takeFirst();
+    QCOMPARE( spy1UpdateWidgetSignals.at( 0 ).toInt(), m_intervals[TINY_BREAK_DURATION] );
+
+    // Part two, waiting out break.
+    QSignalSpy spy2UpdateWidget( &timer, SIGNAL(updateWidget(int)) );
+    QSignalSpy spy2Minimize( &timer, SIGNAL(minimize()) );
+
+    for ( int i = 0; i < m_intervals[TINY_BREAK_DURATION]; i++ ) {
+        QCOMPARE( timer.m_state, RSITimer::TimerState::Resting );
+        idle_time_ptr->setIdleTime( ( i + 1 ) * 1000 );
+        timer.timeout();
+    }
+    QCOMPARE( timer.m_state, RSITimer::TimerState::Monitoring );
+    QCOMPARE( spy2Minimize.count(), 1 );
+    QCOMPARE( spy2UpdateWidget.count(), m_intervals[TINY_BREAK_DURATION] - 1 );
+    for ( int i = 1; i < m_intervals[TINY_BREAK_DURATION]; i++ ) {
+        QList<QVariant> spy2UpdateWidgetSignals = spy2UpdateWidget.takeFirst();
+        QCOMPARE( spy2UpdateWidgetSignals.at( 0 ).toInt(), m_intervals[TINY_BREAK_DURATION] - i );
+    }
+    QCOMPARE( spyEndShortBreak.count(), 1 );
+}
+
+void RSITimerTest::regularBreaks()
+{
+    std::unique_ptr<RSIIdleTimeFake> idle_time( new RSIIdleTimeFake() );
+    RSIIdleTimeFake* idle_time_ptr = idle_time.get();
+    RSITimer timer( std::move( idle_time ), m_intervals, true, false );
+
+    QSignalSpy spyEndShortBreak( &timer, SIGNAL( endShortBreak( void ) ) );
+    QSignalSpy spyEndLongBreak( &timer, SIGNAL( endLongBreak( void ) ) );
+
+    int tinyBreaks = m_intervals[BIG_BREAK_INTERVAL] / ( m_intervals[TINY_BREAK_INTERVAL] + m_intervals[PATIENCE_INTERVAL] + m_intervals[TINY_BREAK_DURATION] );
+    int tick = 0;
+
+    for ( int j = 0; j < tinyBreaks; j++ ) {
+        // Tiny break, mix of activity and idleness till small break.
+        QSignalSpy spyRelax( &timer, SIGNAL(relax(int,bool)) );
+        QSignalSpy spyUpdateIdleAvg( &timer, SIGNAL(updateIdleAvg(double)) );
+
+        for ( int i = 0; i < m_intervals[TINY_BREAK_INTERVAL]; ++i, ++tick ) {
+            QCOMPARE( timer.m_state, RSITimer::TimerState::Monitoring );
+            if ( i % 2 == 0 ) {
+                idle_time_ptr->setIdleTime( 0 );
+            } else {
+                idle_time_ptr->setIdleTime( 1000 );
+            }
+            timer.timeout();
+        }
+
+        for ( int i = 0; i < m_intervals[TINY_BREAK_DURATION]; ++i, ++tick ) {
+            // No activity during break -- obeying.
+            QCOMPARE( timer.m_state, RSITimer::TimerState::Suggesting );
+            idle_time_ptr->setIdleTime( ( i + 1 ) * 1000 );
+            timer.timeout();
+        }
+        QCOMPARE( timer.m_state, RSITimer::TimerState::Monitoring );
+    }
+
+    // Expected ticks till big break, accounting for pauses.
+    int ticks = m_intervals[BIG_BREAK_INTERVAL] + tinyBreaks * m_intervals[TINY_BREAK_DURATION];
+    for ( int j = tick; j < ticks; j++ ) {
+        QCOMPARE( timer.m_state, RSITimer::TimerState::Monitoring );
+        if ( j % 2 == 0 ) {
+            idle_time_ptr->setIdleTime( 0 );
+        } else {
+            idle_time_ptr->setIdleTime( 1000 );
+        }
+        timer.timeout();
+    }
+    for ( int i = 0; i < m_intervals[BIG_BREAK_DURATION]; i++ ) {
+        // No activity during break -- obeying.
+        QCOMPARE( timer.m_state, RSITimer::TimerState::Suggesting );
+        idle_time_ptr->setIdleTime( ( i + 1 ) * 1000 );
+        timer.timeout();
+    }
+    QCOMPARE( timer.m_state, RSITimer::TimerState::Monitoring );
+
+    QCOMPARE( spyEndShortBreak.count(), tinyBreaks );
+    QCOMPARE( spyEndLongBreak.count(), 1 );
 }
 
 #include "rsitimer_test.moc"
